#!/usr/local/bin/php
<?php

/******************************************************************
*                     Xoops Language Checker                      *
*                                 by GIJ=CHECKMATE                *
*                                    http://www.peak.ne.jp/       *
******************************************************************/


define( 'FOR_XOOPS_LANG_CHECKER' , 1 ) ;

// recurrence function
function crawl_dir( $base_dir , $sub_dir )
{
	global $verbose_mode , $lang_files ;

	// Check the directory can be opened
	if( ! ( $dir = @opendir( "$base_dir$sub_dir" ) ) ) {
		if( $verbose_mode ) echo "open error (path=$base_dir$sub_dir).\n" ;
		return ;
	}

	while( ( $file = readdir( $dir ) ) !== false ) {
		// Ignore files started with '.'
		if( ord( $file ) == 0x2E ) continue ;
		// If the file is a directory, recur it.
		else if( is_dir( "$base_dir$sub_dir/$file" ) ) crawl_dir( $base_dir , "$sub_dir/$file" ) ;
		// Add only php files into the array.
		else if( substr( $file , -4 ) == ".php" ) {
			$lang_files[] = "$sub_dir/$file" ;
		}
	}
	closedir( $dir ) ;
}


// Main initialization
$lang_files = array() ;
$dst_langs = array() ;
$verbose_mode = false ;
$patch_mode = false ;
$default_src_lang = 'english' ;
$patcher_exec = 'xoops_patch_langs.php' ;
$differ_exec = 'xoops_diff_langs.php' ;
$usage = 
"usage: %s language_dir [base_language] [target_language ...]
  -h, --help      show this
      --patch     append missing define() written in base lang into the target
  -v, --verbose   explain what is being done
 '$default_src_lang' is selected if base_language is blank
 all languages are selected if target_language is blank
 Author: GIJ=CHECKMATE <gij@peak.ne.jp>  http://www.peak.ne.jp/
" ;


// treatment with parameters and options
$params = array() ;
$lastslash = strrpos( $argv[0] , '/' ) ;
$myexecname = substr( $argv[0] , $lastslash + 1 ) ;
$myexecdir = substr( $argv[0] , 0 , $lastslash ) ;
foreach( $argv as $param ) {
	if( ord( $param ) == 0x2d ) {
		switch( $param ) {
			case '--help' :
			case '-h' :
				printf( $usage , $myexecname ) ;
				exit ;
			case '--verbose' :
			case '-v' :
				$verbose_mode = true ;
				break ;
			case '--patch' :
				$patch_mode = true ;
				break ;
		}
	} else {
		$params[] = $param ;
	}
}

// base directory
if( empty( $params[1] ) ) {
	$lang_dir = "." ;
} else {
	$lang_dir = $params[1] ;
	if( substr( $lang_dir , -1 ) == '/' ) $lang_dir = substr( $lang_dir , 0 , -1 ) ;
}

// source language
if( empty( $params[2] ) ) {
	$src_lang = $default_src_lang ;
} else {
	$src_lang = strtolower( $params[2] ) ;
}

// Check if source directory exists
if( ! is_dir( "$lang_dir/$src_lang" ) ) {
	$lang_dir .= "/language" ;
	if( ! is_dir( "$lang_dir/$src_lang" ) ) {
		die( "Can't open the language directory '$lang_dir/$src_lang'.\n" ) ;
	}
}

// dest languages
if( empty( $params[3] ) ) {
	if( ! ( $dir = @opendir( "$lang_dir" ) ) ) {
		echo "open error (path=$lang_dir).\n" ;
		exit ;
	}
	while( ( $file = readdir( $dir ) ) !== false ) {
		// Ignore files started with '.'
		if( ord( $file ) == 0x2E ) continue ;
		// If the file is a directory, add it into dst_langs.
		else if( is_dir( "$lang_dir/$file" ) ) $dst_langs[] = $file ;
	}
	closedir( $dir ) ;
} else {
	$i = 3 ;
	while( isset( $params[ $i ] ) ) {
		$dst_langs[] = strtolower( $params[ $i ] ) ;
		$i ++ ;
	}
}

// crawl a source directory
crawl_dir( "$lang_dir/$src_lang" , "" ) ;
if( sizeof( $lang_files ) <= 0 ) die( "No php file found.\n" ) ;

// crawl destination directories
foreach( $dst_langs as $dst_lang ) {
	if( $verbose_mode ) echo "Language $dst_lang:\n" ;
	foreach( $lang_files as $lang_file ) {
		if( $verbose_mode ) echo "  Checking $lang_file:\n" ;
		if( $dst_lang == 'ja_utf8' ) {
			$src_file = "$lang_dir/japanese$lang_file" ;
			$src_mail_dir = "$lang_dir/japanese/mail_template" ;
			$src_help_dir = "$lang_dir/japanese/help" ;
		} else {
			$src_file = "$lang_dir/$src_lang$lang_file" ;
			$src_mail_dir = "$lang_dir/$src_lang/mail_template" ;
			$src_help_dir = "$lang_dir/$src_lang/help" ;
		}
		$dst_file = "$lang_dir/$dst_lang$lang_file" ;
		$dst_mail_dir = "$lang_dir/$dst_lang/mail_template" ;
		$dst_help_dir = "$lang_dir/$dst_lang/help" ;
		if( is_readable( $dst_file ) ) {
			if( $patch_mode ) {
				system( "$myexecdir/$patcher_exec $src_file $dst_file" ) ;
			} else {
				system( "$myexecdir/$differ_exec $src_file $dst_file" ) ;
			}
		} else {
			echo "    $dst_file doesn't exist\n" ;
			if( $patch_mode ) {
				if( $dst_lang == 'ja_utf8' && $src_lang == 'japanese' ) {
					system( "/usr/bin/iconv --from-code=EUC-JP --to-code=UTF-8 $src_file >$dst_file" ) ;
					echo "      success to convert from $src_file to $dst_file\n" ;
				} else {
					if( ! @copy( $src_file , $dst_file ) ) {
						echo "      failed to create $dst_file\n" ;
					} else {
						echo "      success to copy from $src_file to $dst_file\n" ;
					}
				}
			}
		}
	}

	if( $patch_mode && is_dir( $src_mail_dir ) && ! is_dir( $dst_mail_dir ) ) {
		// mail_template/*.tpl
		mkdir( $dst_mail_dir ) ;
		$dh = opendir( $src_mail_dir ) ;
		while( $file = readdir( $dh ) ) {
			if( substr( $file , 0 , 1 ) == '.' ) continue ;
			system( "/usr/bin/iconv --from-code=EUC-JP --to-code=UTF-8 $src_mail_dir/$file >$dst_mail_dir/$file" ) ;
		}
		closedir( $dh ) ;
	}

	if( $patch_mode && is_dir( $src_help_dir ) && ! is_dir( $dst_help_dir ) ) {
		// help/*.html
		mkdir( $dst_help_dir ) ;
		$dh = opendir( $src_help_dir ) ;
		while( $file = readdir( $dh ) ) {
			if( substr( $file , 0 , 1 ) == '.' ) continue ;
			system( "/usr/bin/iconv --from-code=EUC-JP --to-code=UTF-8 $src_help_dir/$file >$dst_help_dir/$file" ) ;
		}
		closedir( $dh ) ;
	}

	if( $verbose_mode ) echo "\n" ;
}

?>
